const { app, BrowserWindow, dialog, Menu, shell } = require('electron');
const Sentry = require('@sentry/electron');
const comet = require('@sternenbauer/comet-electron');
const path = require('path');

const runBroker = require('./broker');

// initialize sentry
Sentry.init({
  dsn: 'https://f0c11a1be9c1461ea801e83e3bb2b735@o123327.ingest.sentry.io/5462160',
});

// handle updates
comet('_comet_desktop.shiftr.io', (err) => {
  Sentry.captureException(err);
}).then();

function setupMenu() {
  // check if mac
  const isMac = process.platform === 'darwin';

  // build menu
  const menu = Menu.buildFromTemplate([
    ...(isMac ? [{ role: 'appMenu' }] : []),
    { role: 'fileMenu' },
    { role: 'editMenu' },
    { role: 'viewMenu' },
    { role: 'windowMenu' },
    {
      role: 'help',
      submenu: [
        ...(!isMac ? [{ role: 'about' }] : []), // TODO: Not available on linux.
        {
          label: 'Visit shiftr.io...',
          click() {
            shell.openExternal('https://www.shiftr.io').then();
          },
        },
      ],
    },
  ]);

  // set menu
  Menu.setApplicationMenu(menu);
}

// the http port
let httpPort;

function startBroker(port, done) {
  // run broker
  runBroker(
    port,
    () => {
      // set http port
      httpPort = port + 1;

      // signal
      done();
    },
    (error) => {
      if (error === 'used-ports') {
        dialog
          .showMessageBox({
            type: 'error',
            buttons: ['Use Alternative Ports', 'Quit'],
            defaultId: 0,
            title: 'Ports already in use',
            message: 'The required ports are already in use.',
            detail: `Another application is using the port ${port} or ${port + 1}.`,
          })
          .then((ret) => {
            // check quit
            if (ret.response === 1) {
              app.quit();
            }

            // retry
            startBroker(port + 1000, done);
          });
      } else {
        dialog
          .showMessageBox({
            type: 'error',
            buttons: ['Quit'],
            title: 'Unexpected Error',
            message: error,
          })
          .then(() => {
            app.quit();
          });
      }
    }
  );
}

function showConsole() {
  // check port
  if (!httpPort) {
    return;
  }

  // create window
  const window = new BrowserWindow({
    width: 1200,
    height: 800,
    title: 'shiftr.io Desktop',
    icon: path.join(app.getAppPath(), 'assets', 'icon.png'),
  });

  // prepare console
  const consoleURL = `http://localhost:${httpPort}`;

  // handle new windows
  window.webContents.on('new-window', (e, url) => {
    e.preventDefault();
    shell.openExternal(url).then();
  });

  // handle navigations
  window.webContents.on('will-navigate', (e, url) => {
    if (url !== consoleURL) {
      e.preventDefault();
      shell.openExternal(url).then();
    }
  });

  // load console
  window.loadURL(consoleURL).then();
}

// await ready
app.on('ready', () => {
  // setup menu
  setupMenu();

  // start broker
  startBroker(1883, () => {
    // show console
    showConsole();
  });
});

// quit when all windows are closed (Windows, Linux)
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', () => {
  // recreate window if missing (macOS)
  if (BrowserWindow.getAllWindows().length === 0) {
    showConsole();
  }
});
